"""
This module contains TimeObject implementation.

There is no need to import this module directly. Use the methods of
the L{Ruleset object <ruleset.Ruleset>} to create and manipulate
TimeObjects.

    >>> o = r.object_add("/TimeObjects/Test")
    >>> o.add("200905090000-200909140000")
    >>> o.add("Mon,Tue,Wed,Thu,Fri@0800-1600")
    >>> print o.items
    ['200905090000-200909140000', 'Mon,Tue,Wed,Thu,Fri@0800-1600']

@var __revision__: Revision identifier.
"""

__revision__ = '%K%'

import plobject
from thisversion import PLDB
import plutil
from time import strptime, mktime, strftime, gmtime, localtime, timezone, altzone

def _mk_dayint(days):
	"""
	This function is hilarious..
	"""

	i = 0
	for d in days.split(","):
		d = d.strip()
		if d == "Mon":
			i |= PLDB.TIMEOBJECT_DAYMASK_MON
		elif d == "Tue":
			i |= PLDB.TIMEOBJECT_DAYMASK_TUE
		elif d == "Wed":
			i |= PLDB.TIMEOBJECT_DAYMASK_WED
		elif d == "Thu":
			i |= PLDB.TIMEOBJECT_DAYMASK_THU
		elif d == "Fri":
			i |= PLDB.TIMEOBJECT_DAYMASK_FRI
		elif d == "Sat":
			i |= PLDB.TIMEOBJECT_DAYMASK_SAT
		elif d == "Sun":
			i |= PLDB.TIMEOBJECT_DAYMASK_SUN
		else:
			raise ValueError("Got something I didn't want to in the day str '%s'"% d)
	return i

class TimeObject(plobject.PLObject):
	"""
	TimeObjects Implementation

	Object path: /TimeObjects

	@cvar Item: TimeObject Item implementation.
	"""

	__slots__ = ('_prop', '_pldb', '_callback', '_item_added')
	_may_exclude = True

	base_path = plutil.roConst("/TimeObjects", "Only the type portion of the path")
	type = plutil.roConst(PLDB.OBJECT_TYPE_TIMEOBJECT, "Number representing the type of object")

	def _create_item(self, value):

		item = self.Item(self)

		at = value.split("@")
		if len(at) == 2:
			# This is a DAYTIME thingy.
			days = at[0]
			time = at[1]
			item._prop.type = PLDB.TIMEOBJECT_ITEM_TYPE_DAYTIME
			item._prop.days = _mk_dayint(days)
			v = time.split("-")
			if not len(v) == 2:
				raise ValueError("Malformed time string (expected a dash in there)")
			
			#if len(v[0]) not in [3,4] or len(v[1]) not in [3,4]: # This also allows one digit hour, e.g '745-945'
			if len(v[0]) != 4 or len(v[1]) != 4:
				raise ValueError("Malformed time string (expected each time in format HHmm)")

			try:
				""" Use time in localtime """
				item._prop.value1 = int(v[0][:-2])*60 + int(v[0][-2:])
				item._prop.value2 = int(v[1][:-2])*60 + int(v[1][-2:])
			except:
				raise ValueError("Malformed time string (expected in format HHmm-HHmm, with HH as 00-24 and mm as 00-59)")

			if int(v[0][-2:]) > 59 or int(v[1][-2:]) > 59 or int(v[0][:-2]) > 23 or int(v[1][:-2]) > 24 or int(v[1]) > 2400:
				raise ValueError("Malformed time string (expected in format HHmm-HHmm, with HH as 00-24 and mm as 00-59)")

			if item._prop.value2 < item._prop.value1:
				raise ValueError("Last time in range is smaller than first time.")
				
		else:
			item._prop.type = PLDB.TIMEOBJECT_ITEM_TYPE_DATERANGE
			item._prop.days = 0
			# this is a RANGE thingy format YYYYMMDDHHmm-YYYYMMDDHHmm
			d = value.split("-")
			if not len(d) == 2:
				raise ValueError("The string was not formatted correctly")
			try:
				""" Use time in localtime """
				ts1 = mktime(strptime(d[0], "%Y%m%d%H%M"))
				ts2 = mktime(strptime(d[1], "%Y%m%d%H%M"))
				item._prop.value1 = ts1 - [timezone, altzone][localtime(ts1)[8] == 1]
				item._prop.value2 = ts2 - [timezone, altzone][localtime(ts2)[8] == 1]
			except:
				raise ValueError("Malformed time string")


		return item

	def add(self, value, exclude=False):
		"""
		Adds a TimeObject item to this TimeObject.

		@type value: C{str}
		@param value: This is a identifier for the item value. The type of the item
			will be automatically detected from the string. The item can either be
			a date range with the format I{YYYYMMDDHHmm-YYYYMMDDHHmm} or a
			specification of days and hours of a week. This format is:
			I{days@HHmm-HHmm} where days is one or more of Mon, Tue, Wed, Thu, Fri, Sat, Sun
			separated by comma. I.e. I{Wed,Fri@1000-1100} means every Wednesday and
			Friday between 10:00 and 11:00.

		@type exclude: C{boolean}
		@param exclude: If this is set to True the item will be a exclude
			Item instead of a include Item.

		"""
		plobject.PLObject.add(self, value, exclude)


	class Item(plobject.PLObject.Item):
		"""
		TimeObject Item implementation.
		Should not be instantiated, use L{TimeObject.add} instead.

		@type exclude: C{boolean}
		@type type: C{int}
		@type days: C{int}
		@type value1: C{int}
		@type value2: C{int}
		"""
		__slots__ = ('_prop', '_parent')
		_Prop = plutil.Prop(exclude=bool, type=int, days=int, value1=int, value2=int, *plobject._item_props)

		def __str__(self):
			days = []
			i = self._prop.days
			if self._prop.type == PLDB.TIMEOBJECT_ITEM_TYPE_DAYTIME:
				if i & PLDB.TIMEOBJECT_DAYMASK_MON:
					days.append("Mon")
				if i & PLDB.TIMEOBJECT_DAYMASK_TUE:
					days.append("Tue")
				if i & PLDB.TIMEOBJECT_DAYMASK_WED:
					days.append("Wed")
				if i & PLDB.TIMEOBJECT_DAYMASK_THU:
					days.append("Thu")
				if i & PLDB.TIMEOBJECT_DAYMASK_FRI:
					days.append("Fri")
				if i & PLDB.TIMEOBJECT_DAYMASK_SAT:
					days.append("Sat")
				if i & PLDB.TIMEOBJECT_DAYMASK_SUN:
					days.append("Sun")

				nstr = "%0.2d%0.2d-%0.2d%0.2d" % ((self._prop.value1 / 60), (self._prop.value1 % 60),
						      (self._prop.value2 / 60), (self._prop.value2 % 60))

				return "%s%s@%s" % (['', '!'][self._prop.exclude], ",".join(days), nstr)

			elif self._prop.type == PLDB.TIMEOBJECT_ITEM_TYPE_DATERANGE:
				return "%s%s-%s" % (['', '!'][self._prop.exclude],
						    strftime("%Y%m%d%H%M", gmtime(self._prop.value1)),
						    strftime("%Y%m%d%H%M", gmtime(self._prop.value2)))
			else:
				raise ValueError("Unsupported type")

		type = property(lambda self: self._prop.type, doc="Internal number representing the item type (day/time or date-range)")
		days = property(lambda self: self._prop.days, doc="Bitmap of days")
		exclude = property(lambda self: self._prop.exclude, doc="True = exclude, False = include")
		value1 = property(lambda self: self._prop.value1, doc="From-time")
		value2 = property(lambda self: self._prop.value2, doc="To-time")
