"""
This module contains StatisticsObject implementation.

There is no need to import this module directly. Use the methods of
the L{Ruleset object <ruleset.Ruleset>} to create and manipulate
StatisticsObjects.

    >>> o = r.statisticsobject_add('Test Statistics')

@var __revision__: Revision identifier.
"""

__revision__ = '%K%'

from packetlogic2.exceptions import PLDBUnsupportedInFirmware

class StatisticsObject(object):
	"""
	Base class for StatisticsObjects.

	This class should not be instantiated and should instead be created and
	accessed through:

		- L{Ruleset.statisticsobject_add}
		- L{Ruleset.statisticsobject_find}
		- L{Ruleset.statisticsobject_find_id}
		- L{Ruleset.statisticsobject_list}
		- L{Ruleset.statisticsobject_remove}

	Class variables are read-only, use methods to alter the object.

	"""
	__slots__ = ('_prop', '_pldb')

	def __init__(self, _pldb=None, **props):
		raise PLDBUnsupportedInFirmware("Statistics objects require firmware v12.0 or newer")

	def rename(self, newname):
		"""
		Rename the current object

		I{destroys netobject cache}

		@type newname: C{str}
		@param newname: The name that the object should have after rename process

		"""
		raise PLDBUnsupportedInFirmware("Statistics objects require firmware v12.0 or newer")
		
	def _machineid(self):
		""" Use of property 'machineid' is deprecated since v11.0beta5, use 'systemid' instead."""
		import warnings
		warnings.warn("Use of property 'machineid' is deprecated, use 'systemid' instead.",
			      DeprecationWarning, stacklevel=2)
		return self._prop.systemid

	def change(self, *args, **kvargs):
		raise NotImplementedError("Modification of statisticsobjects not implemented yet")
	
	machineid = property(_machineid)

	id = property(lambda self: self._prop.id, doc="id number of this object")
	name = property(lambda self: self._prop.name, doc="Name of the statistics object")

	total_fields = property(lambda self: self._prop.total_fields, doc="Total fields")
	graph_fields = property(lambda self: self._prop.graph_fields, doc="Graph fields")
	graph_freq = property(lambda self: self._prop.graph_freq, doc="Graph freq")

	min_bytes_in = property(lambda self: self._prop.min_bytes_in, doc="Min bytes in")
	min_bytes_out = property(lambda self: self._prop.min_bytes_out, doc="Min bytes out")

	no_root = property(lambda self: self._prop.no_root, doc="NetObject root")
	so_root = property(lambda self: self._prop.so_root, doc="ServiceObject root")

	split = property(lambda self: self._prop.split, doc="Split string")

	creation_date = property(lambda self: self._prop.creation_date, doc="Date and time when this object was created")
	modification_date = property(lambda self: self._prop.modification_date, doc="Date and time when this object was last modified")
	creator = property(lambda self: self._prop.creator, doc="Username that created this object")
	modified_by = property(lambda self: self._prop.modified_by, doc="Username that last modified this object")

	systemid = property(lambda self: self._prop.systemid, doc="System ID of the PacketLogic this object belongs to. (Replaces machineid)")
	is_global = property(lambda self: self._prop.is_global, doc="True if this object resides on another system (always false in 12.1)")
