"""
This module contains functionality for reading the connection log from
the PacketLogic system.

Do NOT import this module directly. The L{PLConnection
<packetlogic2.PLConnection>} object should be used, e.g:

    >>> import packetlogic2
    >>> pl = packetlogic2.connect("192.168.1.25", "admin", "pldemo00")
    >>> cl = pl.ConnLog()

@var __revision__: Revision identifier.
"""

__revision__ = '%K%'

import time, socket
from thisversion import PLDB
import resource
import plutil
import struct

class ConnLog(resource.Resource):
	"""
	This resource will allow you to search the connectionlog
	(if enabled in the system).
	"""
	def _resource(self):
		return PLDB.RESOURCE_CONNSEARCH

	def search(self, timefr, timeto, server=None,
	           serverport=None, client=None,
	           clientport=None, service=None,
	           serverhostname=None, protocol=None,
	           max_hits=50, offset=0, callback=None):

		"""
		Search the connection log with parameters. All parameters except
		timefr and timeto can be omitted, which will evaluate to "any".
		You must specify at least one other search criteria though.

		Time is specified as the local time on the PacketLogic system.
		The 'time' item in the returned dictionary is a unix timestamp
		adjusted for the timezone on the PacketLogic system, to get a
		python timetuple representing the PacketLogic systems local
		time convert it (despite the misleading function name) with:
		pl_local_time_tuple = time.gmtime(connlog_data['time'])

		Known limitations: Specifying a short duration (less than 10
		minutes, may result in an empty result set despite data being
		available. This might be resolved in future versions.
		
			>>> x.search("2005-12-19 10:00", "2005-12-20 12:04", service="http", max_hits=3)
			[{'client': '10.2.20.75',
			  'client_port': 3429,
			  'protocol': 6,
			  'server': '194.14.70.50',
			  'server_port': 80,
			  'serverhostname': 'www.svt.se',
			  'service': 'http',
			  'time': 1135033194},
			 {'client': '10.2.95.71',
			  'client_port': 1958,
			  'protocol': 6,
			  'server': '70.85.224.194',
			  'server_port': 80,
			  'serverhostname': 'www.dumpalink.com',
			  'service': 'http',
			  'time': 1135033194},
			 {'client': '10.2.20.75',
			  'client_port': 3430,
			  'protocol': 6,
			  'server': '194.14.70.50',
			  'server_port': 80,
			  'serverhostname': 'www.svt.se',
			  'service': 'http',
			  'time': 1135033194}]

		Omitting hours:minutes in date/times, and the addition of 59
		seconds to the to-time, is new since version 11.3beta5.

		Using datetime.date[time] is new since version 11.3beta5.

		@type timefr: C{str} or datetime.date or datetime.datetime
		@param timefr: Start the search at this date, the date has to
			be in I{YYYY-MM-DD HH:MM} or I{YYYY-MM-DD} format,
			time default to 00:00.

		@type timeto: C{str} or datetime.date or datetime.datetime
		@param timeto: Stop the search at this date, the date has to
			be in I{YYYY-MM-DD HH:MM} or I{YYYY-MM-DD} format,
			time default to 23:59, also 59 seconds is assumed always.

		@type server: C{str}
		@param server: The servers ip address.

		@type serverport: C{int}
		@param serverport: The server port.

		@type client: C{str}
		@param client: The clients ip address

		@type clientport: C{int}
		@param clientport: client port.

		@type service: C{str}
		@param service: a string with the name of the service (i.e. I{ftp} or I{ftp data})

		@type serverhostname: C{str}
		@param serverhostname: the resolved hostname of the server, this only
			works on protocols where the serverhostname is included like I{http}.

		@type protocol: C{int}
		@param protocol: The IP protocol used. (6=TCP)

		@type max_hits: C{int}
		@param max_hits: Number of answers you want.

		@type offset: C{int}
		@param offset: Start on this number of answers .. (i.e. if you set max_hits to 50
			and offset to 50 you will get answers between 50 and 100)

		@type callback: C{callable}
		@param callback: Optional function to be called for each chunk
		                 of data received instead of returning a list.

		@rtype: C{list} of C{dict}
		@returns: A list of hits unless callback is set.

		@raise ValueError: If search criteria is missing.
		@raises PLDBUnsupportedFeatureError: If 'connlog' feature is disabled (See L{ruleset.Ruleset.features_get()}).
		"""

		self._require_features('connlog')

		if not [x for x in (server, serverport,
		                    client, clientport,
		                    service, serverhostname,
		                    protocol) if x is not None]:
			raise ValueError("Need at least one search criteria.")

		if server is not None:
			server = struct.unpack("!l", socket.inet_aton(server))[0]
		if client is not None:
			client = struct.unpack("!l", socket.inet_aton(client))[0]


		# In case the caller uses datetime objects, we convert them to iso-date strings
		timefr = str(timefr)
		timeto = str(timeto)

		# Even if undocumented, it will allow single-digit month, date and hour, hence the ranges for the lengths.
		if 8 <= len(timefr) <= 10:
			timefr = time.mktime(time.strptime(timefr, "%Y-%m-%d"))
		elif 13 <= len(timefr) <= 16:
			timefr = time.mktime(time.strptime(timefr, "%Y-%m-%d %H:%M"))
		elif len(timefr) == 19: #single digit not allowed here, this is for str(datetime.datetime(...))
			timefr = time.mktime(time.strptime(timefr, "%Y-%m-%d %H:%M:%S"))
		else:
			raise ValueError("Date/time should be in format 'YYYY-MM-DD' or 'YYYY-MM-DD HH:MM'.")
		
		if 8 <= len(timeto) <= 10:
			timeto = time.mktime(time.strptime(timeto + " 23:59:59", "%Y-%m-%d %H:%M:%S"))
		elif 13 <= len(timeto) <= 16:
			timeto = time.mktime(time.strptime(timeto + ":59", "%Y-%m-%d %H:%M:%S"))
		elif len(timeto) == 19:
			timeto = time.mktime(time.strptime(timeto, "%Y-%m-%d %H:%M:%S"))
		else:
			raise ValueError("Date/time should be in format 'YYYY-MM-DD' or 'YYYY-MM-DD HH:MM'.")
		
		# mktime transforms the time to UTC but we want to send it in a localtime-adjusted timestamp.
		# (We wouldn't have to if time.mktime was replaced by calendar.timegm, but that's yet another module.
		timefr -= [time.timezone, time.altzone][bool(time.localtime(timefr)[8])]
		timeto -= [time.timezone, time.altzone][bool(time.localtime(timeto)[8])]

		cid = self.session.connsearch_search(timefr, timeto,
						     server, serverport,
						     client, clientport,
						     service, serverhostname,
						     protocol, max_hits,
						     offset)

		class CB(plutil._Callback):
			def __init__(self, pldb):
				plutil._Callback.__init__(self, pldb)
				self.prev = None
			def handleOK(self):
				if self.prev is not None:
					return self.prev.convert(dict)
				return []
			def handleData(self, msg):
				l = self._pldb.session.connsearch_search_handle (msg, self.prev)
				if callback is not None:
					callback(l.convert(dict))
				else:
					self.prev = l

		return plutil._do(self, cid, CB(self))
