"""
This module contains NetObject implementation.

There is no need to import this module directly. Use the methods of
the L{Ruleset object <ruleset.Ruleset>} to create and manipulate
NetObjects.

    >>> o = r.object_add("/NetObjects/Test")
    >>> o.add("192.168.20.0/255.255.255.0")
    >>> o.add("192.168.25.10-192.168.25.20")
    >>> o.add("192.168.26.1")
    >>> print o.items
    ['192.168.20.0/255.255.255.0', '192.168.25.10-192.168.25.20', '192.168.26.1']

@var __revision__: Revision identifier.
"""

__revision__ = '%K%'

import struct
from socket import inet_ntoa
import plobject
from thisversion import PLDB
import plutil

class NetObjectItem(plobject.PLObject.Item):
	"""
	Netobject Item implementation.
	Should not be instantiated, use L{NetObject.add} instead.

	@type type: C{int}
	@type exclude: C{bool}
	@type value1: C{str}
	@type value2: C{str}
	"""
	__slots__ = ('_prop', '_parent')
	_Prop = plutil.Prop(exclude=bool, type=int, value1=str, value2=str, *plobject._item_props)

	def __str__(self):
		if self._prop.exclude:
			ex = "!"
		else:
			ex = ""

		if self._prop.type == PLDB.NETOBJECT_ITEM_TYPE_IPV4_ADDRESS:
			return ex + self._prop.value1
		elif self._prop.type == PLDB.NETOBJECT_ITEM_TYPE_IPV4_NETWORK:
			return ex + "%s/%s" % (self._prop.value1, self._prop.value2)
		elif self._prop.type == PLDB.NETOBJECT_ITEM_TYPE_IPV4_RANGE:
			return ex + "%s-%s" % (self._prop.value1, self._prop.value2)
		else:
			raise ValueError("Wrong item-type for this object")

	type = property(lambda self: self._prop.type, doc="Internal number representing the item type (single, network, or range)")
	exclude = property(lambda self: self._prop.exclude, doc="True = exclude, False = include")
	value1 = property(lambda self: self._prop.value1, doc="IP address, first IP address if part of a range or prefix if it's a network.")
	value2 = property(lambda self: self._prop.value2, doc="Empty string, last IP address if part of a range or netmask if it's a network.")


class NetObject(plobject.PLObject):
	"""
	NetObject Implementation

	Object path: /NetObjects

	@cvar Item: NetObject Item implementation.
	"""

	Item = NetObjectItem

	__slots__ = ('_prop', '_pldb', '_callback', '_item_added')
	_may_exclude = True

	base_path = plutil.roConst("/NetObjects", "Only the type portion of the path")
	type = plutil.roConst(PLDB.OBJECT_TYPE_NETOBJECT, "Number representing the type of object")

	def _create_item(self, value):

		item = NetObjectItem(self)

		if not value.find("/") == -1:
			# This is type network!
			s = value.split("/")
			if not len(s) == 2:
				raise ValueError("Format is not correct for this item type")

			item._prop.type = PLDB.NETOBJECT_ITEM_TYPE_IPV4_NETWORK
			item._prop.value1, item._prop.value2 = s

			if not plutil.verify_ip(item._prop.value1):
				raise ValueError("First argument doesn't seem like a IP address")
			# if we have no ".", its short form. convert to a full subnet mask.
			if item._prop.value2.find(".") == -1 and item._prop.value2.isdigit():
				if int(item._prop.value2) < 0 or int(item._prop.value2) > 32:
					raise ValueError("Second argument is not a proper prefix length between 0 and 32.")
				item._prop.value2 = inet_ntoa(struct.pack("L", 0xFFFFFFFF >> (32 - int(item._prop.value2))))
			if not plutil.verify_ip(item._prop.value2):
				raise ValueError("Second argument doesn't seem like a subnet mask")
		elif not value.find("-") == -1:
			# type range!
			s = value.split("-")
			if not len(s) == 2:
				raise ValueError("Format is not correct for this item type")
			
			item._prop.type = PLDB.NETOBJECT_ITEM_TYPE_IPV4_RANGE
			item._prop.value1, item._prop.value2 = s

			if not plutil.verify_ip(item._prop.value1) or not plutil.verify_ip(item._prop.value2):
				raise ValueError("First argument doesn't seem like a IP address")

		else:
			# type address
			item._prop.type = PLDB.NETOBJECT_ITEM_TYPE_IPV4_ADDRESS
			item._prop.value1 = value
			item._prop.value2 = None

			if not plutil.verify_ip(item._prop.value1):
				raise ValueError("First argument doesn't seem like a IP address")

		return item

	def add(self, value, exclude=False):
		"""
		Adds a NetObject item to this NetObject.

		@type value: C{str}
		@param value: The value is a string describing the NetObject item. The type of the item
			will be automatically detected from the string. The following syntax is valid:
				- IPv4 Address, Just give the plain IPaddress I{xxx.xxx.xxx.xxx}
				- IPv4 Network, Give the network address with the netmask after I{xxx.xxx.xxx.xxx/yyy.yyy.yyy.yyy}
				- IPv4 CIDR notation, Give the network address and prefix length separated with slash I{xxx.xxx.xxx.xxx/nn} (note that this will be converted to network/netmask notation internally)
				- IPv4 Range, Give the start and stop address with a dash as delimiter I{xxx.xxx.xxx.xxx-yyy.yyy.yyy.yyy}

		@type exclude: C{boolean}
		@param exclude: If this is set to True the item will be a exclude
			Item instead of a include Item.

		"""
		plobject.PLObject.add(self, value, exclude)


