"""
This module contains ShapingObject implementation.

There is no need to import this module directly. Use the methods of
the L{Ruleset object <ruleset.Ruleset>} to create and manipulate
ShapingObjects.

    >>> so = r.shapingobject_add('Test Shaping',
                                 inbound=[(0, 0, 1024)], # 1024 kbyte/s inbound
                                 outbound=[(0, 0, 512)]) # 512 kbyte/s outbound

@var __revision__: Revision identifier.
"""

__revision__ = '%K%'

import plutil

from packetlogic2.exceptions import PLDBUnsupportedInFirmware

NONEXISTANT_SPLIT_TYPE = object()

def _shapingobjectlist(pldb):
	cid = pldb.session.shaping_object_list()
	return plutil._do(pldb, cid, plutil._ListCB(pldb, pldb.session.shaping_object_list_handle, ShapingObject))

def _shapingobjectadd(_pldb, name, vbs_hours, inbound, outbound, bidir, split, max_connections, flags):
	kwargs = locals().copy()

	flags = ShapingObjectFlags(flags)

	cid = _pldb.session.shaping_object_add(name, vbs_hours, (inbound, outbound, bidir), split, max_connections, int(flags))
	(id, tstamp) = plutil._do(_pldb, cid, plutil._AddCB(_pldb, _pldb.session.shaping_object_add_handle))
	res = ShapingObject(id = id,
			    creation_date = tstamp,
			    is_global = False,
			    modification_date = tstamp,
			    creator = _pldb.user,
			    modified_by = _pldb.user,
			    systemid = _pldb.systemid,
			    **kwargs)
	return res

def _shapingobjectupdate(_pldb, obj, name, vbs_hours, inbound, outbound, bidir, split, max_connections, intflags):
	#v = locals()
	cid = _pldb.session.shaping_object_modify(obj.id, name, vbs_hours, (inbound, outbound, bidir), split, max_connections, intflags)
	tstamp = plutil._do(_pldb, cid, plutil._AddCB(_pldb, _pldb.session.shaping_object_modify_handle))
	obj._prop.modification_date = tstamp
	obj._prop.modified_by = _pldb.user
	#[setattr(obj._prop, var, val) for var, val in v.iteritems() if var not in ["_pldb", "obj", "intflags"]]

def _rename(pldb, obj, newname):
	cid = pldb.session.shaping_object_rename(obj.id, newname)
	return plutil._do(pldb, cid, plutil._AddCB(pldb, pldb.session.shaping_object_rename_handle))

def _remove(pldb, obj):
	cid = pldb.session.shaping_object_remove(obj.id)
	return plutil._do(pldb, cid, plutil._AddCB(pldb, pldb.session.shaping_object_remove_handle))

_Prop_ShapingObjectFlags = plutil.Prop(brown=bool, host_fairness=bool, counter=bool)
class ShapingObjectFlags(plutil.BaseFlags):
	""" Used internaly to represent the flags on a L{ShapingObject}.

	@type brown: C{bool}
	@type host_fairness: C{bool}
	@type counter: C{bool}
	
	"""
	__slots__ = ('_prop')
	_Prop_class = _Prop_ShapingObjectFlags

	def _set_flags_other_type(self, intflags):
		""" Support bitmap int to set flags (when creted from API list)."""
		if isinstance(intflags, int):
			self._prop.update(brown=intflags & 0x01, host_fairness=intflags & 0x02, counter=intflags & 0x10)
		else:
			plutil.BaseFlags._set_flags_other_type(self, intflags)
			
	def __int__(self):
		return self._prop.brown * 0x01 + \
		    self._prop.host_fairness * 0x02 + \
		    self._prop.counter * 0x10

	brown = property(lambda self: self._prop.brown, doc="Enable the BROWN Shaping Algoritm")
	host_fairness = property(lambda self: self._prop.host_fairness, doc="Enable fairness between hosts")
	counter = property(lambda self: self._prop.counter, doc="Enable byte counter")
			
_Prop_ShapingObject = plutil.Prop('name', 'vbs_hours', 'inbound', 'outbound', 'bidir', 'split', 'max_connections',
				  'id', 'creation_date', 'is_global', 'modification_date', 'creator', 'modified_by', 'systemid',
				  flags = ShapingObjectFlags)
class ShapingObject(object):
	"""
	Base class for ShapingObjects.

	This class should not be instantiated and should instead be created and
	accessed through:

		- L{Ruleset.shapingobject_add}
		- L{Ruleset.shapingobject_find}
		- L{Ruleset.shapingobject_find_id}
		- L{Ruleset.shapingobject_list}
		- L{Ruleset.shapingobject_remove}

	Class variables are read-only, use methods to alter the object.

	@type bidir: list
	@type outbound: list
	@type inbound: list
	@type name: str
	@type split: int
	@type vbs_hours: int
	@type max_connections: int
	@type creation_date: str
	@type creator: str
	@type id: int
	@type systemid: str
	@type modification_date: str
	@type modified_by: str
	@type is_global: C{boolean}
	@type flags: object
	@type machineid: str
	"""
	__slots__ = ('_prop', '_pldb')

	def __init__(self, _pldb=None, **props):
		self._pldb = _pldb
		self._prop = _Prop_ShapingObject(**props)

	def __repr__(self):
		return "<PacketLogic ShapingObject ('%s', inbound=%s, outbound=%s, bidir=%s, split=%d, vbs_hours=%d, max_connections=%d, flags=%s)>" % (
			self._prop.name, self._prop.inbound, self._prop.outbound, self._prop.bidir, self._prop.split,
			self._prop.vbs_hours, self._prop.max_connections, str(list(self._prop.flags)))

	def change_limits(self, inbound=None, outbound=None, bidir=None):
		"""
		Change the limits in the shaping object. Any unspecified
		limits are left as is.

		@type inbound: C{list} of C{tuple}
		@param inbound: list of limits for incoming traffic, where
		each limit is (max_transfer, max_packets,
		max_speed) max_transfer must be 0
		(unlimited) on the last entry in the list.
		Speed is specified in kbyte/s and transfer in bytes.

		@type outbound: C{list} of C{tuple}
		@param outbound: same as inbound but for outgoing traffic

		@type bidir: C{list} of C{tuple}
		@param bidir: same as inbound but for both directions

		"""

		if inbound is None:
			inbound = self.inbound
		elif self.vbs_hours == 0 and len(inbound) != 1:
			raise ValueError("Limit list must be of length 1 when vbs_hours is 0 (inbound)")
		elif inbound[-1][0] != 0:
			raise ValueError("last limit must have max_transfer=0 (inbound)")
			
		if outbound is None:
			outbound = self.outbound
		elif self.vbs_hours == 0 and len(outbound) != 1:
			raise ValueError("Limit list must be of length 1 when vbs_hours is 0 (outbound)")
		elif outbound[-1][0] != 0:
			raise ValueError("last limit must have max_transfer=0 (outbound)")

		if bidir is None:
			bidir = self.bidir
		elif self.vbs_hours == 0 and len(bidir) != 1:
			raise ValueError("Limit list must be of length 1 when vbs_hours is 0 (bidir)")
		elif bidir[-1][0] != 0:
			raise ValueError("last limit must have max_transfer=0 (bidir)")

		_shapingobjectupdate(self._pldb, self, self._prop.name, self._prop.vbs_hours, inbound, outbound, bidir,
				     self._prop.split, self._prop.max_connections, int(self._prop.flags))

		self._prop.inbound = inbound
		self._prop.outbound = outbound
		self._prop.bidir = bidir

	def change(self, vbs_hours=None, split=None, max_connections=None): # inbound=None, outbound=None, bidir=None,
		"""
		Change properties of the shaping object. Any unspecified
		(or set to C{None}) property will be left as is.

		@type vbs_hours: C{int}
		@param vbs_hours: Duration in hours if this is a VBS (Volume Based Shaping) object.
				  Value 0 means that VBS is not used on this object.
				  Value 1 means that no sliding window is used, and you will need
				  to reset volume manually with L{Realtime.vbs_reset <pld.PLD.vbs_reset>}.
				  
				  You can't change vbs_hours if it is non-zero.

		@type split: C{int}
		@param split: Type of split, one of: L{Ruleset <ruleset.Ruleset>}.SPLIT_NONE,
		              SPLIT_LOCALHOST, SPLIT_HOST_NETOBJECT,
		              SPLIT_LOCAL_NETOBJECT, SPLIT_SERVER_NETOBJECT,
		              SPLIT_CLIENT_NETOBJECT, SPLIT_CONNECTION,
			      SPLIT_SUBSCRIBER

			      You can't change split if vbs_hours is non-zero.

		@type max_connections: C{int}
		@param max_connections: Limit number of connections, or 0 for unlimited.

		@raises ValueError: if split or vbs_hours is changed on a VBS object.

		"""

		#if inbound is None:
		inbound = self.inbound
		#if outbound is None:
		outbound = self.outbound
		#if bidir is None:
		bidir = self.bidir
		
		if vbs_hours is None:
			vbs_hours = self.vbs_hours

		elif self.vbs_hours and vbs_hours != self.vbs_hours:
			raise ValueError, "vbs_hours can't be changed when it is non-zero"
			
		if split is None:
			split = self.split
		elif split is NONEXISTANT_SPLIT_TYPE:
			raise PLDBUnsupportedInFirmware("Specified split is not supported in this firmware version")

		elif self.vbs_hours and split != self.split:
			raise ValueError, "split can't be changed when vbs_hours is non-zero"
			
		if max_connections is None:
			max_connections = self.max_connections

		_shapingobjectupdate(self._pldb, self, self._prop.name, vbs_hours, inbound, outbound, bidir,
				     split, max_connections, int(self._prop.flags))

		#self._prop.inbound = inbound
		#self._prop.outbound = outbound
		#self._prop.bidir = bidir
		self._prop.vbs_hours = vbs_hours
		self._prop.split = split
		self._prop.max_connections = max_connections

	def set_flags(self, *reset, **flags):
		"""
		Set or reset one or more of the flags of the shaping object.
		You may also reset all flags with the single argument None, or clone an existing flags object.
		You may combine them.

		If only keyword argument(s) are used, only the specified flags will be affected.
		Possible flags are: brown, host_fairness.
		Use a trueish value to set the flag, or falseish value to reset it.

			>>> shobj.set_flags(brown=True, host_fairness=False)
			>>> shobj.flags
			ShapingObjectFlags(brown=True)
			
			>>> shobj.set_flags(None)
			>>> shobj.flags
			ShapingObjectFlags(None)

			>>> shobj.set_flags(another_shobj.flags)
			>>> shobj.flags
			ShapingObjectFlags(host_fairness=True)
			
			>>> shobj.set_flags(None, brown=True)
			>>> shobj.flags
			ShapingObjectFlags(brown=True)
			
		"""
		if len(reset) == 1 and isinstance(reset[0], ShapingObject):
			# Might as well support supplying a shapingobject as argument too.
			self._prop.flags._set_flags(reset[0].flags, **flags)

		else:
			self._prop.flags._set_flags(*reset, **flags)

		_shapingobjectupdate(self._pldb, self, self._prop.name, self._prop.vbs_hours, self._prop.inbound, self._prop.outbound,
				     self._prop.bidir, self._prop.split, self._prop.max_connections, int(self._prop.flags))

	def rename(self, newname):
		"""
		Rename the current object

		I{destroys netobject cache}

		@type newname: C{str}
		@param newname: The name that the object should have after rename process

		"""

		_rename(self._pldb, self, newname)
		self._prop.name = newname

		self._pldb._invalidate_cache()

	def _attr_set(self, key, value):
		if self._prop.is_global:
			raise ValueError("Can't set attributes on nonlocal objects")
		ci = self._pldb.session.shapingobject_attr_set(self._prop.id, key, value)
		c = plutil._AddCB(self._pldb)
		plutil._do(self._pldb, ci, c)

	def _attr_remove(self, key):
		if self._prop.is_global:
			raise ValueError("Can't remove attributes from nonlocal objects")
		ci = self._pldb.session.shapingobject_attr_remove(self._prop.id, key)
		c = plutil._AddCB(self._pldb)
		plutil._do(self._pldb, ci, c)

	def _attr_list(self):
		ci = self._pldb.session.shapingobject_attr_list(self._prop.systemid, self._prop.id)
		c = plutil._SimpleListCB(self._pldb, self._pldb.session.shapingobject_attr_list_handle)
		c.list = []
		res = dict(plutil._do(self._pldb, ci, c))
		return plutil.AttributesDict(res, self._attr_set, self._attr_remove)
		
	def _machineid(self):
		""" Use of property 'machineid' is deprecated since v11.0beta5, use 'systemid' instead."""
		import warnings
		warnings.warn("Use of property 'machineid' is deprecated, use 'systemid' instead.",
			      DeprecationWarning, stacklevel=2)
		return self._prop.systemid
	
	machineid = property(_machineid)

	name = property(lambda self: self._prop.name, doc="Name of the shaping object")
	vbs_hours = property(lambda self: self._prop.vbs_hours, doc="Duration of a volume-based ShapingObject")
	inbound = property(lambda self: self._prop.inbound, doc="Inbound shaping")
	outbound = property(lambda self: self._prop.outbound, doc="Outbound shaping")
	bidir = property(lambda self: self._prop.bidir, doc="Bidirectional shaping")
	split = property(lambda self: self._prop.split, doc="Split by")
	max_connections = property(lambda self: self._prop.max_connections, doc="Maximum number of allowed connections")
	flags = property(lambda self: self._prop.flags, doc="Object containing the flags (if any) set on this ShapingObject.")
	id = property(lambda self: self._prop.id, doc="id number of this object")
	creation_date = property(lambda self: self._prop.creation_date, doc="Date and time when this object was created")
	is_global = property(lambda self: self._prop.is_global, doc="True if this object resides on another system")
	modification_date = property(lambda self: self._prop.modification_date, doc="Date and time when this object was last modified")
	creator = property(lambda self: self._prop.creator, doc="Username that created this object")
	modified_by = property(lambda self: self._prop.modified_by, doc="Username that last modified this object")
	systemid = property(lambda self: self._prop.systemid, doc="System ID of the PacketLogic this object belongs to. (Replaces machineid)")

	_attributes_doc = """
	Arbitrary key/value attributes stored for this shapingobject.

	    >>> o.attributes
	    {}
	    >>> o.attributes['test'] = "Arbitrary string value"
	    >>> o.attributes
	    {'test': "Arbitrary string value"}

	@raises PLDBUnsupportedInFirmware: If attributes is accessed without firmware support.

 	@attention: Attributes requires v12.2 firmware or newer.
	"""
	attributes = property(_attr_list, doc=_attributes_doc)

